#!/usr/bin/env python

"""
Check user memory usage according to a given threshold.
"""

import sys
import os
import argparse
from datetime import datetime, timedelta

from slapos.collect.db import Database

def escapeSqlStringValue(string):
  return string.replace("'", "\\'")

def getMemoryInfo(database_path, moment, user):
  """
  Gives `user` memory info dict at `moment`.
  ex: for moment = 2017-08-13 01:02:03
      it searches the results from 01:00:02 to 01:02:03
      and take the latest one.
  """
  date_str = moment.strftime("%Y-%m-%d")
  # time is `(moment-1min) +/- 1min`
  time_from_str = (moment - timedelta(minutes=2) - timedelta(seconds=1)).strftime("%H:%M:%S")
  time_to_str = moment.strftime("%H:%M:%S")
  if time_from_str > time_to_str:
    #   "23:59:59" > "00:02:00"
    time_from_str = "00:00:00"
  escaped_user = escapeSqlStringValue(user)
  memory_info = {}
  database = Database(database_path, create=False, timeout=5)
  try:
    database.connect()
    result = list(database.select(
      table="user",
      date=date_str,
      columns="memory_percent, memory_rss",
      where="partition = '{}' AND time BETWEEN '{}' AND '{}'".format(escaped_user, time_from_str, time_to_str),
      order="time DESC",
      limit=1,
    ))
    if len(result) == 0:
      return (None, "No result found in collector.db.")
    memory_info["percent"] = result[0][0]
    memory_info["byte"] = int(result[0][1])  # in byte
  finally:
    database.close()

  return (memory_info, "")

def checkMemoryUsage(database_path, moment, user, threshold, unit="byte"):
  if unit not in ("byte", "percent"):
    raise ValueError("invalid unit")
  memory_info, error = getMemoryInfo(database_path, moment, user)
  if error:
    return (False, "error", error)
  if unit == "byte":
    if memory_info["byte"] <= threshold:
      return (True, "User memory usage: {byte}B ({percent:.1f}%)".format(**memory_info), "")
    return (False, "High user memory usage: {byte}B ({percent:.1f}%)".format(**memory_info), "")
  else:  # if unit == "percent":
    if memory_info["percent"] <= threshold:
      return (True, "User memory usage: {percent:.1f}% ({byte}B)".format(**memory_info), "")
    return (False, "High user memory usage: {percent:.1f}% ({byte}B)".format(**memory_info), "")

def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("--collectordb", required=True,
                      help="the directory path of the 'collector.db' file.")
  parser.add_argument("--user", required=True)
  parser.add_argument("--threshold", required=True, type=float)
  parser.add_argument("--unit", default="byte", choices=("byte", "percent"))
  args = parser.parse_args()

  database_path = args.collectordb
  # --collectordb : can also be the path of 'collector.db' itself
  if os.path.basename(database_path) == "collector.db":
    database_path = os.path.dirname(database_path)

  result, message, error = checkMemoryUsage(
    database_path,
    datetime.now(),
    user=args.user,
    threshold=args.threshold,
    unit=args.unit,
  )
  if error:
    print error
    return 0
  print message
  return 0 if result else 1

if __name__ == "__main__":
  sys.exit(main())



import argparse
import sys
import os
import signal

def killpidfromfile():
  """deprecated: use below kill() instead"""
  if len(sys.argv) != 3:
    raise ValueError('Invocation: %s <pidfile> <signal name>' % sys.argv[0])
  file = sys.argv[1]
  sig = getattr(signal, sys.argv[2], None)
  if sig is None:
    raise ValueError('Unknown signal name %s' % sys.argv[2])
  pid = int(open(file).read())
  print 'Killing pid %s with signal %s' % (pid, sys.argv[2])
  os.kill(pid, sig)

def sublist(a, b):
  try:
    i = a.index(b[0])
  except IndexError:
    return True
  except ValueError:
    return False
  return a[i:i+len(b)] == b

def kill():
  parser = argparse.ArgumentParser()
  _ = parser.add_argument
  _('--exe', action='append',
    help="match against executable path")
  _('-f', '--full', action='store_true',
    help="match positional arguments against the full command line")
  _('-F', '--pidfile', action='append',
    help="read PID's from file")
  _('-n', '--name', action='append',
    help="match against process name")
  _('-s', '--signal', required=True,
    help="signal to send to each matched process")
  _('arg', nargs='*',
    help="match against command line")
  args = parser.parse_args()

  try:
    s = args.signal.split('+', 1)
    s = getattr(signal, "SIG" + s[0]) + (len(s) > 1 and int(s[1]))
  except (AttributeError, ValueError):
    parser.error('Unknown signal name %s' % args.signal)

  pid = args.pidfile and [int(open(p).read()) for p in args.pidfile]
  exe = args.exe and map(os.path.realpath, args.exe)

  import psutil
  r = 1
  me = os.getpid()
  for p in psutil.process_iter():
    try:
      if (me == p.pid or
          pid and p.pid not in pid or
          args.name and p.name() not in args.name or
          exe and p.exe() not in exe):
        continue
      cmdline = p.cmdline()
      if cmdline == args.arg if args.full else sublist(cmdline, args.arg):
        p.send_signal(s)
        print 'killed pid %s with signal %s' % (p.pid, args.signal)
        r = 0
    except psutil.Error:
      pass
  return r



# Command-line script to generate a RSS feed from a bunch of well-formated
# JSON items.
# This script tries to be the more generic possible. The items used to generate
# the feed must be JSON-formatted (because of simplicity to read/write them),
# and their keys must follow the names of elements of items as described
# in the RSS2 specification :
#   http://cyber.law.harvard.edu/rss/rss.html#hrelementsOfLtitemgt


import argparse
import collections
import datetime
import json
import os
import PyRSS2Gen as rss

def parseArguments():
  parser = argparse.ArgumentParser()
  parser.add_argument('--output', dest='output', type=str, required=True,
                      help='Path where to save the file')
  parser.add_argument('--status-item-path', dest='status_item_path',
                      type=str, required=True,
                      help='Path where to find feed items')
  parser.add_argument('--max-item', dest='max_item', type=int,
                      default=50, required=False,
                      help='Maximum number of items in the feed')

  parser.add_argument('--title', dest='feed_title', type=str, required=True,
                      help='Title of the feed')
  parser.add_argument('--link', dest='feed_link', type=str, required=True,
                      help='Link of the feed')
  parser.add_argument('--description', dest='feed_description',
                      type=str, required=False,
                      help='Description of the feed')

  option = parser.parse_args()
  if not hasattr(option, 'feed_description'):
    option.feed_description = option.feed_title

  return option


def deleteFileList(file_list):
  for file in file_list:
    try:
      os.unlink(file)
    except OSError:
      pass


def getRSSItemListFromItemDict(item_dict):
  rss_item_list = []

  for item in item_dict:
    item_dict[item]['pubDate'] = datetime.datetime.fromtimestamp(item_dict[item]['pubDate'])
    rss_item_list.append(rss.RSSItem(**item_dict[item]))

  return rss_item_list


def generateFeed(option):
  item_dict = {} # {file: content}

  for filename in os.listdir(option.status_item_path):
    file_path = os.path.join(option.status_item_path, filename)
    with open(file_path, 'r') as fd:
      try:
        item_dict[file_path] = json.load(fd)
      except ValueError:
        # JSON couldn't be decoded, let's trash it as
        # no useful information can be extracted
        os.unlink(file_path)

  sorted_item_dict = collections.OrderedDict(
      sorted(item_dict.items(), key=lambda x: x[1]['pubDate']))

  # Reduces feed if number of items exceeds max_item
  if len(item_dict) > option.max_item:
    outdated_key_list = sorted_item_dict.keys()[:-option.max_item]
    for outdated_key in outdated_key_list:
      del sorted_item_dict[outdated_key]
    deleteFileList(outdated_key_list)

  # Generate feed
  feed = rss.RSS2(
    title=option.feed_title,
    link=option.feed_link,
    description=option.feed_description,
    lastBuildDate = datetime.datetime.now(),
    items = getRSSItemListFromItemDict(sorted_item_dict)
  )

  return feed.to_xml()


def main():
  option = parseArguments()
  feed = generateFeed(option)
  open(option.output, 'w').write(feed)


if __name__ == "__main__":
  main()



import re
import time
import sys
import gzip
import argparse
import os

r = re.compile("^(\[[^\]]+\]) (\[[^\]]+\]) (.*)$")

def test(log_file, maximum_delay):
  error_amount = 0
  no_route_error = 0
  network_is_unreacheable = 0
  timeout = 0
  parsing_failure = 0
  if not os.path.exists(log_file):
    # file don't exist, nothing to check
    return "OK"

  with open(log_file) as f:

    f.seek(0, 2)
    block_end_byte = f.tell()
    f.seek(-min(block_end_byte, 4096), 1)

    data = f.read()
    for line in reversed(data.splitlines()):
      m = r.match(line)
      if m is None:
        continue
      dt, level, msg = m.groups()
      try:
        try:
          t = time.strptime(dt[1:-1], "%a %b %d %H:%M:%S %Y")
        except ValueError:
          # Fail to parser for the first time, try a different output.
          t = time.strptime(dt[1:-1], "%a %b %d %H:%M:%S.%f %Y")
      except ValueError:
          # Probably it fail to parse
          if parsing_failure < 3:
            # Accept failure 2 times, as the line can be actually
            # cut on the middle.
            parsing_failure += 1
            continue
          raise

      if maximum_delay and (time.time()-time.mktime(t)) > maximum_delay:
        # no result in the latest hour
        break

      if level != "[error]":
        continue

      # Classify the types of errors
      if "(113)No route to host" in msg:
        no_route_error += 1
      elif "(101)Network is unreachable" in msg:
        network_is_unreacheable += 1
      elif "(110)Connection timed out" in msg:
        timeout += 1

      error_amount += 1

  if error_amount:
    return "ERROR=%s (NOTROUTE=%s, UNREACHEABLENET=%s, TIMEOUT=%s)" % (error_amount, no_route_error, network_is_unreacheable, timeout)

  return "OK"

def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("-l", "--log-file", required=True)
  parser.add_argument("-d", "--maximum-delay", default=0)
  args = parser.parse_args()

  log_file = args.log_file

  result = test(args.log_file, args.maximum_delay)

  print result
  if result != "OK":
    sys.exit(1)




# -*- coding: utf-8 -*-
# vim: set et sts=2:
# pylint: disable-msg=W0311,C0301,C0103,C0111


import os
import re
import shutil

from git import Repo
from flask import jsonify


def cloneRepo(url, workDir, user="", email=""):
  """Clone a repository
  Args:
    workDir is the path of the new project
    url is the url of the repository to be cloned
    email is the user's email
    user is the name of the user"""

  if not workDir:
    return jsonify(code=0,
                   result="Can not create project folder.")

  if os.path.exists(workDir) and len(os.listdir(workDir)) < 2:
    shutil.rmtree(workDir)  # delete useless files
  repo = Repo.clone_from(url, workDir)
  config_writer = repo.config_writer()
  config_writer.add_section("user")
  if user != "":
    config_writer.set_value("user", "name", user.encode("utf-8"))
  if email != "":
    config_writer.set_value("user", "email", email)

def updateGitConfig(repository, user, email):
  if not os.path.exists(repository):
    return
  repo = Repo(repository)
  config_writer = repo.config_writer()
  if user != "":
    config_writer.set_value("user", "name", user)
  if email != "":
    config_writer.set_value("user", "email", email)
  config_writer.release()

def gitStatus(project):
  """Run git status and return status of specified project folder
  Args:
    project: path of the projet to get status
  Returns:
    a list with (result of git status, current branch, isdirty)"""

  repo = Repo(project)
  git = repo.git
  result = git.status().replace('#', '')
  branch = git.branch().replace(' ', '').split('\n')
  isdirty = repo.is_dirty(untracked_files=True)
  return (result, branch, isdirty)


def switchBranch(project, branch):
  """Switch a git branch
  Args:
    project: directory of the local git repository
    name: switch from current branch to `name` branch"""

  repo = Repo(project)
  current_branch = repo.active_branch.name
  if branch == current_branch:
    return False
  else:
    git = repo.git
    git.checkout(branch)
    return True


def addBranch(project, name, onlyCheckout=False):
  """Add new git branch to the repository
  Args:
    project: directory of the local git repository
    name: name of the new branch
    onlyCheckout: if True then the branch `name` is created before checkout
  Returns:
    True or False"""
  
  if not os.path.exists(project):
    return False
  repo = Repo(project)
  git = repo.git
  if not onlyCheckout:
    git.checkout('-b', name)
  else:
    git.checkout(name)
  return True


def getDiff(project):
  """Get git diff for the specified project directory"""
  result = ""
  try:
    repo = Repo(project)
    git = repo.git
    current_branch = repo.active_branch.name
    result = git.diff(current_branch)
  except Exception as e:
    result = safeResult(str(e))
  return result

def gitCommit(project, msg):
  """Commit changes for the specified repository
  Args:
    project: directory of the local repository
    msg: commit message"""
  code = 0
  json = ""
  repo = Repo(project)
  if repo.is_dirty:
    git = repo.git
    #add file to be commited
    files = repo.untracked_files
    for f in files:
      git.add(f)
    #Commit all modified and untracked files
    git.commit('-a', '-m', msg)
    code = 1
  else:
    json = "Nothing to be commited"
  return jsonify(code=code, result=json)

def gitPush(project):
  """Push changes for the specified repository
  Args:
    project: directory of the local repository
    msg: commit message"""
  code = 0
  json = ""
  try:
    repo = Repo(project)
    git = repo.git
    #push changes to repo
    current_branch = repo.active_branch.name
    git.push('origin', current_branch)
    code = 1
  except Exception as e:
    json = safeResult(str(e))
  return jsonify(code=code, result=json)


def gitPull(project):
  result = ""
  code = 0
  try:
    repo = Repo(project)
    git = repo.git
    git.pull()
    code = 1
  except Exception as e:
    result = safeResult(str(e))
  return jsonify(code=code, result=result)


def safeResult(result):
  """Parse string and remove credential of the user"""
  regex = re.compile("(https:\/\/)([\w\d\._-]+:[\w\d\._-]+)\@([\S]+\s)", re.VERBOSE)
  return regex.sub(r'\1\3', result)



##############################################################################
#
# Copyright (c) 2017 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os.path
import tempfile
import shutil
from datetime import date

from slapos.apachedex import build_command

class TestApachedexCommand(unittest.TestCase):
  def setUp(self):
    self.apachedex = "/bin/apachedex"
    self.today = date.today().strftime("%Y-%m-%d")
    _, self.acesslog1 = tempfile.mkstemp()
    _, self.acesslog2 = tempfile.mkstemp()

  def test_simpleCommand(self): 
    
    command = build_command(self.apachedex,
                            'foo.html',
                            [self.acesslog1, self.acesslog2],
                            '--default foo')
    self.assertEqual(command, ['/bin/apachedex',
                               '--js-embed',
                               '--out', 'foo.html',
                               '--default', 'foo',
                               '--error-detail', self.acesslog1, self.acesslog2 ])
 
  def test_complexCommand(self):
    command = build_command(self.apachedex,
                            'bar.html',
                            [self.acesslog1, self.acesslog2],
                            '--base bar foo --default foo')

    self.assertEqual(command, ['/bin/apachedex',
                               '--js-embed',
                               '--out', 'bar.html',
                               '--base', 'bar', 'foo',
                               '--default', 'foo',
                               '--error-detail', self.acesslog1, self.acesslog2 ])
  def test_raiseErro(self):
    self.assertRaises(ValueError, build_command, self.apachedex, 'foo.html', [])
if __name__ == '__main__':
  unittest.main()



import errno
import glob
import os
import subprocess
import sys

from contextlib import contextmanager
from hashlib import sha256
from zc.buildout.configparser import parse


@contextmanager
def CwdContextManager(path):
  """
  Context Manager for executing code in a given directory.
  There is no need to provide fallback or basic checks
  in this code, as these checkes should exist in the code
  invoking this Context Manager.
  If someone needs to add checks here, I'm pretty sure
  it means that they are trying to hide legitimate errors.
  See tests to see examples of invokation
  """
  old_path = os.getcwd()
  try:
    os.chdir(path)
    yield
  finally:
    os.chdir(old_path)


def getExcludePathList(path):
  excluded_path_list = [
    "*.sock",
    "*.socket",
    "*.pid",
    ".installed*.cfg",
  ]

  def append_relative(path_list):
    for p in path_list:
      p = p.strip()
      if p:
        excluded_path_list.append(os.path.relpath(p, path))

  for partition in glob.glob(os.path.join(path, "instance", "slappart*")):
    if not os.path.isdir(partition):
      continue

    with CwdContextManager(partition):
      try:
        with open("srv/exporter.exclude") as f:
          exclude = f.readlines()
      except IOError as e:
        if e.errno != errno.ENOENT:
          raise
      else:
        append_relative(exclude)
      for installed in glob.glob(".installed*.cfg"):
        try:
          with open(installed) as f:
            installed = parse(f, installed)
        except IOError as e:
          if e.errno != errno.ENOENT:
            raise
        else:
          for section in installed.itervalues():
            append_relative(section.get(
              '__buildout_installed__', '').splitlines())

  return excluded_path_list


def getSlappartSignatureMethodDict():
  slappart_signature_method_dict = {}
  for partition in glob.glob("./instance/slappart*"):
    if os.path.isdir(partition):
      script_path = os.path.join(partition, 'srv', '.backup_identity_script')
      if os.path.exists(script_path):
        slappart_signature_method_dict[partition] = script_path
  return slappart_signature_method_dict


def read_file_by_chunk(path, chunk_size=1024 * 1024):
  with open(path, 'rb') as f:
    chunk = f.read(chunk_size)
    while chunk:
      yield chunk
      chunk = f.read(chunk_size)


def getSha256Sum(file_path_list):
  result_list = []

  for file_path in file_path_list:
    hash_sum = sha256()
    for chunk in read_file_by_chunk(file_path):
      hash_sum.update(chunk)
    result_list.append("%s  %s" % (hash_sum.hexdigest(), file_path))

  return result_list


def writeSignatureFile(slappart_signature_method_dict, runner_working_path, signature_file_path='backup.signature'):
  special_slappart_list = slappart_signature_method_dict.keys()
  signature_list = []

  for dirpath, dirname_list, filename_list in os.walk('.'):
    if dirpath == '.' or not filename_list:
      continue

    # Find if special signature function should be applied
    signature_process = None
    for special_slappart in special_slappart_list:
      backup_identity_script_path = os.path.join(
        runner_working_path,
        slappart_signature_method_dict[special_slappart]
      )

      if dirpath.startswith('./' + os.path.relpath(os.path.join('./runner', special_slappart))):
        signature_process = subprocess.Popen(
          backup_identity_script_path,
          stdin=subprocess.PIPE,
          stdout=subprocess.PIPE,
        )
        break

    if signature_process:
      (output, error_output) = signature_process.communicate(
        '\0'.join([os.path.join(dirpath, filename) for filename in filename_list])
      )

      if signature_process.returncode != 0:
        print(
          "An issue occured when calculating the custom signature"
          " with %s :\n%s\n%s" % (
            backup_identity_script_path, output, error_output
          )
        )
        sys.exit(1)

      # We have to rstrip as most programs return an empty line
      # at the end of their output
      signature_list.extend(output.strip('\n').split('\n'))
    else:
      signature_list.extend(
        getSha256Sum([
          os.path.join(dirpath, filename)
          for filename in filename_list
        ])
      )

    # Write the signatures in file
    with open(signature_file_path, 'w+') as signature_file:
      signature_file.write("\n".join(sorted(signature_list)))



##############################################################################
#
# Copyright (c) 2010 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import os
import argparse
import subprocess
import psutil
import glob

def getAgumentParser():
  """
  Return argument parser for secure delete script.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('-n', '--iterations', default=1, type=int,
                      help='overwrite N times instead of the default (1)')
  parser.add_argument('-u', '--remove', action='store_true',
                      help='truncate and remove file after overwriting.')
  parser.add_argument('-z', '--zero', action='store_true',
                      help='add a final overwrite with zeros to hide shredding.')
  parser.add_argument('-s', '--check-exist', action='store_true',
                      default=False,
                      help='Check if files exist. If a file don\t exists, skip instead of raise.')
  parser.add_argument('--file', dest='file_list',
                      required=True, nargs='+', metavar='FILE',
                      help='File(s) to overwrite and remove if -u is used.')
  parser.add_argument('-p', '--check-pid', type=int, metavar="PID",
                      help='If process is running with PID, abort command. ' \
                           'This is to prevent delete files used by a process.')
  parser.add_argument('--check-pid-file', metavar="PID_FILE",
                      help='Same as "check-pid" option but read PID from PID_FILE')
  parser.add_argument('--shred-bin', default="/usr/bin/shred",
                      help='Path of shred binary used to wipe files. Default: %(default)s')

  return parser

def getFileList(source_file_list, check_exist):
  file_list = []
  for file_path in source_file_list:
    if file_path.find('*') != -1 or file_path.find('?') != -1:
      sub_list = glob.glob(file_path)
    else:
      sub_list = [file_path]
    for file in sub_list:
      if check_exist and not os.path.exists(file):
        continue
      file_list.append(file)
      if os.path.islink(file):
        # remove the link target file
        file_list.append(os.path.realpath(file))
  return file_list

def shred(options):
  # Overwrite the specified FILE(s) repeatedly, in order to make it harder
  # for even very expensive hardware probing to recover the data.
  # using Linux `shred` utility

  check_pid = None
  if options.check_pid_file:
    with open(options.check_pid_file, 'r') as fpid:
      check_pid = int(fpid.read())
  else:
    check_pid = options.check_pid
  if check_pid and psutil.pid_exists(check_pid):
    return "check-pid is enabled and process with pid %s is running. Cannot " \
      "wipe file(s) while the process is running." % check_pid

  arg_list = [options.shred_bin, '-n', str(options.iterations), '-v']
  if options.remove:
    arg_list.append('-u')
  if options.zero:
    arg_list.append('-z')
  arg_list.extend(getFileList(options.file_list, options.check_exist))

  pshred = subprocess.Popen(arg_list, stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
  result, stderr = pshred.communicate()
  if pshred.returncode is None:
    pshred.kill()
  if pshred.returncode != 0:
    raise RuntimeError('Command %r failed, with output:\n%s\n%s' % (
      ' '.join(arg_list), result, stderr or ''))
  return result

def main():
  arg_parser = getAgumentParser()
  output = shred(arg_parser.parse_args())
  print output



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2015 Nexedi SA and Contributors. All Rights Reserved.
#                    Hardik Juneja <hardik.juneja@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import os, errno
import subprocess
import argparse
import time
from datetime import date

# run_apachedex.py <apachedex_executable> /srv/etc/output_folder script_name
def build_command(apachedex_executable, output_file,
                  apache_log_list,
                  config = None):

  if not len(apache_log_list):
    raise ValueError("apache_log_list is empty")

  today = date.today().strftime("%Y-%m-%d")
  apachedex = apachedex_executable
  argument_list = [apachedex, '--js-embed', '--out', output_file]

  log_list = []
  for logfile in apache_log_list:
    if not logfile:
      continue
    # Automaticaly replace variable 'date'.
    apache_log = logfile.strip() % {'date': today}
    if not os.path.exists(apache_log):
      print "WARNING: File %s not found..." % apache_log
      continue
    log_list.append(apache_log)
  if not log_list:
    raise ValueError("log_list: no log files to analyse were provided")

  if config:
    config = filter(None, [x.strip() for x in config.split(' ')])
    argument_list += config

  argument_list.append('--error-detail')
  argument_list += log_list

  return argument_list

def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("apachedex_executable", metavar="APACHEDEX_EXECUTABLE")
  parser.add_argument("output_folder", metavar="OUTPUT_FOLDER")
  parser.add_argument("base_url", metavar="BASE_URL")
  parser.add_argument("--apache-log-list", dest="apache_log_list", nargs='*')
  parser.add_argument("--configuration", dest="configuration")
  args = parser.parse_args()

  config = args.configuration
  output_folder = args.output_folder.strip()
  base_url = args.base_url.strip()

  if not os.path.exists(output_folder) or not os.path.isdir(output_folder):
    print "ERROR: Output folder is not a directory. Exiting..."
    return 1

  today = date.today().strftime("%Y-%m-%d")
  output_file = os.path.join(output_folder, 'ApacheDex-%s.html' % today)

  try:
    argument_list = build_command(args.apachedex_executable.strip(),
                            output_file,
                            args.apache_log_list,
                            config)
  except ValueError as e:
    print e
    return 1
  process_handler = subprocess.Popen(argument_list,
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE,
                                     )

  stdout, stderr = process_handler.communicate()
  if process_handler.returncode != 0:
    if stderr:
      print stderr
    return 1
  
  with open(output_file, 'r') as f:
    print base_url + '/ApacheDex-%s.html' % today
  return 0

if __name__ == "__main__":
  sys.exit(main())




# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os
import tempfile
import shutil

from slapos.qemuqmpclient import QemuQMPWrapper, QmpDeviceRemoveError

class TestQemuQMPWrapper(unittest.TestCase):

  def setUp(self):
    self.base_dir = tempfile.mkdtemp()
    self.socket_file = os.path.join(self.base_dir, 'qmp.socket')
    self.call_stack_list = []
    self.free_cpu_slot_amount = 4
    self.hotplugged_memory_amount = 0
    # slot of 1G
    self.memory_slot_size = 1024
    self.event_list = []
    self.fail = False

  def tearDown(self):
    if os.path.exists(self.base_dir):
      shutil.rmtree(self.base_dir)

  def setChange(self, device, value):
    current = self.readChange(device)
    with open(os.path.join(self.base_dir, device), 'w') as f:
      f.write('%s' % (current + value, ))

  def readChange(self, device):
    if os.path.exists(os.path.join(self.base_dir, device)):
      with open(os.path.join(self.base_dir, device)) as f:
        return int(f.read())
    return 0

  def fake_send(self, message, retry=0, sleep=0.5):
    self.call_stack_list.append(message)
    if message.get('execute', '').startswith('query-'):
      return self.returnQueryResult(message)
    elif message.get('execute', '') == 'device_add':
      if message['arguments']['driver'] == 'pc-dimm':
        self.setChange('dimm', self.memory_slot_size)
      elif message['arguments']['driver'] == 'qemu64-x86_64-cpu':
        self.setChange('cpu', 1)
    elif message.get('execute', '') == 'device_del':
      if message['arguments']['id'].startswith('dimm'):
        self.setChange('dimm', -1 * self.memory_slot_size)
      if message['arguments']['id'].startswith('cpu'):
        self.setChange('cpu', -1)
    if self.fail:
      return {"error": {"class": "CommandFailed", "desc": ""}}
    return {"return": {}}

  def fake_getEventList(self, timeout=0, cleanup=False):
    if self.event_list:
      return self.event_list
    else:
      return []

  def returnQueryResult(self, message):
    if message['execute'] == 'query-hotpluggable-cpus':
      # return 4 hotpluggable cpu slots
      hotpluggable_cpu_list = []
      free_cpu_slot = self.free_cpu_slot_amount - self.readChange('cpu')
      for i in range(4, 4 - free_cpu_slot, -1):
        hotpluggable_cpu_list.append({
          u'props': {u'core-id': 0, u'node-id': 0, u'socket-id': i, u'thread-id': 0},
          u'type': u'qemu64-x86_64-cpu',
          u'vcpus-count': 1
        })
      for i in range(4 - free_cpu_slot, 0, -1):
        hotpluggable_cpu_list.append({
          u'props': {u'core-id': 0, u'node-id': 0, u'socket-id': i, u'thread-id': 0},
          u'qom-path': u'/machine/peripheral/cpu%s' % i,
          u'type': u'qemu64-x86_64-cpu',
          u'vcpus-count': 1
        })
      # first cpu
      hotpluggable_cpu_list.append(
        {u'props': {u'core-id': 0, u'node-id': 0, u'socket-id': 0, u'thread-id': 0},
          u'qom-path': u'/machine/unattached/device[0]',
          u'type': u'qemu64-x86_64-cpu',
          u'vcpus-count': 1
        }
      )
      return {"return": hotpluggable_cpu_list}
    elif message['execute'] == 'query-cpus':
      cpu_list = []
      cpu_slot = 4 - self.free_cpu_slot_amount + self.readChange('cpu')
      cpu_list.append({
        u'CPU': 0,
        u'arch': u'x86',
        u'current': True,
        u'halted': True,
        u'pc': -1694470494,
        u'props': {u'core-id': 0, u'node-id': 0, u'socket-id': 0, u'thread-id': 0},
        u'qom_path': u'/machine/unattached/device[0]',
        u'thread_id': 1181
      })
      for i in range(0, cpu_slot):
        cpu_list.append({
          u'CPU': i + 1,
          u'arch': u'x86',
          u'current': False,
          u'halted': True,
          u'pc': -1694470494,
          u'props': {u'core-id': 0, u'node-id': 0, u'socket-id': i + 1, u'thread-id': 0},
          u'qom_path': u'/machine/peripheral/cpu%s' % (i + 1),
          u'thread_id': 1187
        })
      return {"return": cpu_list}
    elif message['execute'] == 'query-memory-devices':
      memory_list = []
      added_mem = self.readChange('dimm') + self.hotplugged_memory_amount
      slot_amount = added_mem / self.memory_slot_size
      for i in range(slot_amount, 0, -1):
        memory_list.append({
          u'data': {
            u'addr': 4563402752,
            u'hotpluggable': True,
            u'hotplugged': True,
            u'id': u'dimm%s' % i,
            u'memdev': u'/objects/mem%s' % i,
            u'node': 0,
            u'size': self.memory_slot_size * 1024 * 1024,
            u'slot': 1
          },
          u'type': u'dimm'
        })
      return {"return": memory_list}
    elif message['execute'] == 'query-memdev':
      memory_list = []
      added_mem = self.readChange('dimm') + self.hotplugged_memory_amount
      slot_amount = added_mem / self.memory_slot_size
      for i in range(slot_amount, 0, -1):
        memory_list.append({
          u'dump': True,
          u'host-nodes': [],
          u'id': u'mem%s' % i,
          u'merge': True,
          u'policy': u'default',
          u'prealloc': False,
          u'size': self.memory_slot_size * 1024 * 1024
        })
      return {"return": memory_list}
    return {"return": {}}

  def test_setVNCPassword(self):
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send

    expected_result = {
      "execute": "change",
      "arguments": {
        "device": "vnc",
        "target": "password",
        "arg": "my password"
      }
    }
    qmpwrapper.setVNCPassword("my password")
    self.assertEqual(len(self.call_stack_list), 1)
    self.assertEqual(self.call_stack_list[0], expected_result)

  def test_updateDevice_cpu_add(self):
    self.free_cpu_slot_amount = 4
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    # add 2 cpu
    cpu_option = {
      'device': 'cpu',
      'amount': 2 + 1
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-hotpluggable-cpus'},
      {
        'execute': 'device_add',
        'arguments': {
          u'socket-id': 1,
          u'thread-id': 0,
          'driver': 'qemu64-x86_64-cpu',
          u'core-id': 0,
          u'node-id': 0,
          'id': 'cpu1'
        }
      },
      {
        'execute': 'device_add',
        'arguments': {
          u'socket-id': 2,
          u'thread-id': 0,
          'driver': 'qemu64-x86_64-cpu',
          u'core-id': 0,
          u'node-id': 0,
          'id': 'cpu2'
        }
      },
      {'execute': 'query-cpus'}
    ]

    self.assertEqual(len(self.call_stack_list), 4)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_cpu_increase(self):
    self.free_cpu_slot_amount = 2
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    # add 2 more cpu
    cpu_option = {
      'device': 'cpu',
      'amount': 4 + 1
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-hotpluggable-cpus'},
      {
        'execute': 'device_add',
        'arguments': {
          u'socket-id': 3,
          u'thread-id': 0,
          'driver': 'qemu64-x86_64-cpu',
          u'core-id': 0,
          u'node-id': 0,
          'id': 'cpu3'
        }
      },
      {
        'execute': 'device_add',
        'arguments': {
          u'socket-id': 4,
          u'thread-id': 0,
          'driver': 'qemu64-x86_64-cpu',
          u'core-id': 0,
          u'node-id': 0,
          'id': 'cpu4'
        }
      },
      {'execute': 'query-cpus'}
    ]

    self.assertEqual(len(self.call_stack_list), 4)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_cpu_remove(self):
    self.free_cpu_slot_amount = 2
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    qmpwrapper.getEventList = self.fake_getEventList
    self.event_list = [{"event": "DEVICE_DELETED"}]
    # add 2 more cpu
    cpu_option = {
      'device': 'cpu',
      'amount': 1 + 1
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-hotpluggable-cpus'},
      {
        'execute': 'device_del',
        'arguments': {
          'id': 'cpu2'
        }
      },
      {'execute': 'query-cpus'}
    ]

    self.assertEqual(len(self.call_stack_list), 3)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_cpu_no_update(self):
    self.free_cpu_slot_amount = 2
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    # keep 2 cpu added
    cpu_option = {
      'device': 'cpu',
      'amount': 2 + 1
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-hotpluggable-cpus'}
    ]

    self.assertEqual(len(self.call_stack_list), 1)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_memory_add(self):
    self.hotplugged_memory_amount = 0
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    # slot of 1G
    self.memory_slot_size = 1024
    # add 2G of RAM = 2 slots to add
    cpu_option = {
      'device': 'memory',
      'nslot': 4,
      'mem': 2048,
      'slot': self.memory_slot_size
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-memory-devices'},
      {'execute': 'query-memdev'},
      {
        'execute': 'object-add',
        'arguments': {
          'id': 'mem1',
          'qom-type': 'memory-backend-ram',
          'props': {'size': self.memory_slot_size * 1024 * 1024}
        }
      },
      {
        'execute': 'device_add',
        'arguments': {
          'driver': 'pc-dimm',
          'id': 'dimm1',
          'memdev': 'mem1'
        }
      },
      {
        'execute': 'object-add',
        'arguments': {
          'id': 'mem2',
          'qom-type': 'memory-backend-ram',
          'props': {'size': 1073741824}
        }
      },
      {
        'execute': 'device_add', 
        'arguments': {
          'driver': 'pc-dimm',
          'id': 'dimm2',
          'memdev': 'mem2'
        }
      },
      {'execute': 'query-memory-devices'}
    ]

    self.assertEqual(len(self.call_stack_list), 7)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_memory_increase(self):
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    self.hotplugged_memory_amount = 2048
    # slot of 1G
    self.memory_slot_size = 1024
    # increase to 3G, add one more slot of 1G
    cpu_option = {
      'device': 'memory',
      'nslot': 4,
      'mem': 3072,
      'slot': self.memory_slot_size
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-memory-devices'},
      {'execute': 'query-memdev'},
      {
        'execute': 'object-add',
        'arguments': {
          'id': 'mem3',
          'qom-type': 'memory-backend-ram',
          'props': {'size': self.memory_slot_size * 1024 * 1024}
        }
      },
      {
        'execute': 'device_add',
        'arguments': {
          'driver': 'pc-dimm',
          'id': 'dimm3',
          'memdev': 'mem3'
        }
      },
      {'execute': 'query-memory-devices'}
    ]

    self.assertEqual(len(self.call_stack_list), 5)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_memory_delete(self):
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    qmpwrapper.getEventList = self.fake_getEventList
    self.event_list = [{"event": "DEVICE_DELETED"}]
    self.hotplugged_memory_amount = 3072
    # slot of 1G
    self.memory_slot_size = 1024
    # decrease memory to 1G, expext remove slot 3 and 2.
    cpu_option = {
      'device': 'memory',
      'nslot': 4,
      'mem': 1024,
      'slot': self.memory_slot_size
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-memory-devices'},
      {'execute': 'query-memdev'},
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm3'}
      },
      {
        'execute': 'object-del',
        'arguments': {'id': u'mem3'}
      },
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm2'}
      },
      {
        'execute': 'object-del',
        'arguments': {'id': u'mem2'}
      },
      {'execute': 'query-memory-devices'}
    ]

    self.assertEqual(len(self.call_stack_list), 7)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_memory_delete_all(self):
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    qmpwrapper.getEventList = self.fake_getEventList
    self.event_list = [{"event": "DEVICE_DELETED"}]
    self.hotplugged_memory_amount = 3072
    # slot of 1G
    self.memory_slot_size = 1024
    # remove all hotplugged memory
    cpu_option = {
      'device': 'memory',
      'nslot': 4,
      'mem': 0,
      'slot': self.memory_slot_size
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-memory-devices'},
      {'execute': 'query-memdev'},
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm3'}
      },
      {
        'execute': 'object-del',
        'arguments': {'id': u'mem3'}
      },
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm2'}
      },
      {
        'execute': 'object-del',
        'arguments': {'id': u'mem2'}
      },
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm1'}
      },
      {
        'execute': 'object-del',
        'arguments': {'id': u'mem1'}
      },
      {'execute': 'query-memory-devices'}
    ]

    self.assertEqual(len(self.call_stack_list), 9)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_memory_no_update(self):
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    self.hotplugged_memory_amount = 3072
    # slot of 1G
    self.memory_slot_size = 1024
    # no changes
    cpu_option = {
      'device': 'memory',
      'nslot': 4,
      'mem': self.hotplugged_memory_amount,
      'slot': self.memory_slot_size
    }
    qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-memory-devices'},
      {'execute': 'query-memdev'}
    ]

    self.assertEqual(len(self.call_stack_list), 2)
    self.assertEqual(self.call_stack_list, expected_result)

  def test_updateDevice_memory_will_reboot(self):
    qmpwrapper = QemuQMPWrapper(self.socket_file, auto_connect=False)
    qmpwrapper._send = self.fake_send
    qmpwrapper.getEventList = self.fake_getEventList
    self.fail = True
    self.hotplugged_memory_amount = 3072
    # slot of 1G
    self.memory_slot_size = 1024
    # decrease memory to 1G, expext remove slot 3 and 2.
    cpu_option = {
      'device': 'memory',
      'nslot': 4,
      'mem': 1024,
      'slot': self.memory_slot_size,
      'canreboot': True
    }
    with self.assertRaises(QmpDeviceRemoveError):
      qmpwrapper.updateDevice(cpu_option)
    expected_result = [
      {'execute': 'query-memory-devices'},
      {'execute': 'query-memdev'},
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm3'}
      },
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm3'}
      },
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm3'}
      },
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm3'}
      },
      {
        'execute': 'device_del',
        'arguments': {'id': u'dimm3'}
      },
      {'execute': 'system_powerdown'},
      {'execute': 'quit'}
    ]

    self.assertEqual(len(self.call_stack_list), 9)
    self.assertEqual(self.call_stack_list, expected_result)

if __name__ == '__main__':
  unittest.main()



from zope import interface as zope_interface
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

import os
import sys

import sqlite3
import argparse
import datetime
import psutil

from slapos.collect.db import Database

class RunPromise(GenericPromise):

  zope_interface.implements(interface.IPromise)

  def __init__(self, config):
    GenericPromise.__init__(self, config)
    # check disk space at least every 3 minutes
    self.setPeriodicity(minute=3)

  def getFreeSpace(self, disk_partition, database, date, time):

    database = Database(database, create=False, timeout=10)
    try:
      # fetch free disk space
      database.connect()
      where_query = "time between '%s:00' and '%s:30' and partition='%s'" % (time, time, disk_partition)
      query_result = database.select("disk", date, "free", where=where_query)
      result = zip(*query_result)
      if not result or not result[0][0]:
        self.logger.info("No result from collector database: disk check skipped")
        return 0
      disk_free = result[0][0]
    except sqlite3.OperationalError, e:
      # if database is still locked after timeout expiration (another process is using it)
      # we print warning message and try the promise at next run until max warn count
      locked_message = "database is locked"
      if locked_message in str(e) and \
          not self.raiseOnDatabaseLocked(locked_message):
        return 0
      raise
    finally:
      try:
        database.close()
      except Exception:
        pass
    return int(disk_free)

  def raiseOnDatabaseLocked(self, locked_message):
    max_warn = 10
    latest_result_list = self.getLastPromiseResultList(result_count=max_warn)
    warning_count = 0
    if len(latest_result_list) < max_warn:
      return False

    for result in latest_result_list[0]:
      if result['status'] == "ERROR" and locked_message in result["message"]:
        return True

    for result_list in latest_result_list:
      found = False
      for result in result_list:
        if result['status'] == "WARNING" and locked_message in result["message"]:
          found = True
          warning_count += 1
          break
      if not found:
        break
    if warning_count == max_warn:
      # too many warning on database locked, now fail.
      return True

    self.logger.warn("collector database is locked by another process")
    return False

  def getInodeUsage(self, path):
    max_inode_usage = 97.99 # < 98% usage
    stat = os.statvfs(path)
    usage_output = ""
    total_inode = stat.f_files
    free_inode = stat.f_ffree
    usage = round((float(total_inode - free_inode) / total_inode), 4) * 100
    if usage > max_inode_usage:
      return "Disk Inodes usages is really high: %s%%" % usage
    elif os.path.exists('/tmp'):
      # check if /tmp is mounted on another disk than path
      tmp_stat = os.statvfs('/tmp')
      if tmp_stat.f_blocks != stat.f_blocks:
        tmp_usage = round((float(tmp_stat.f_files - tmp_stat.f_ffree) / tmp_stat.f_files), 4) * 100
        if tmp_usage > max_inode_usage:
          return "Disk Inodes usage is high: %s%%" % tmp_usage
    return ""


  def sense(self):
    # find if a disk is mounted on the path
    disk_partition = ""
    disk_threshold_file = self.getConfig('threshold-file')
    db_path = self.getConfig('collectordb')
    check_date = self.getConfig('test-check-date')
    path = os.path.join(self.getPartitionFolder(), "") + "extrafolder"
    partitions = psutil.disk_partitions()
    while path is not '/':
      if not disk_partition:
        path = os.path.dirname(path)
      else:
        break
      for p in partitions:
        if p.mountpoint == path:
          disk_partition = p.device
          break
    if not disk_partition:
      self.logger.error("Couldn't find disk partition")
      return

    min_free_size = 1024*1024*1024*2 # 2G by default
    if disk_threshold_file is not None:
      if os.path.exists(disk_threshold_file):
        with open(disk_threshold_file) as f:
          min_size_str = f.read().strip()
          if min_size_str == '0':
            # disable check
            self.logger.info("Free disk space check is disabled\n set a number up to 0 to enable!")
            return
          if min_size_str.isdigit():
            value = int(min_size_str)
            if value >= 200:
              # Minimum value is 200Mb, it's already low
              min_free_size = int(min_size_str)*1024*1024
      else:
        with open(disk_threshold_file, 'w') as f:
          f.write(str(min_free_size/(1024*1024)))

    if check_date:
      # testing mode
      currentdate = check_date
      currenttime = self.getConfig('test-check-time', '09:17')
      disk_partition = self.getConfig('test-disk-partition', '/dev/sda1')
    else:
      # get last minute
      now = datetime.datetime.now()
      currentdate = now.strftime('%Y-%m-%d')
      currenttime = now - datetime.timedelta(minutes=1)
      currenttime = currenttime.time().strftime('%H:%M')

    if db_path.endswith("collector.db"):
      db_path=db_path[:-len("collector.db")]

    free_space = self.getFreeSpace(disk_partition, db_path, currentdate,
                                   currenttime)
    if free_space == 0:
      return
    elif free_space > min_free_size:
      inode_usage = self.getInodeUsage(self.getPartitionFolder())
      if inode_usage:
        self.logger.error(inode_usage)
      else:
        self.logger.info("Disk usage: OK")
      return

    free_space = round(free_space/(1024.0*1024*1024), 2)
    min_space = round(min_free_size/(1024.0*1024*1024), 2)
    self.logger.error('Free disk space low: remaining %s G (threshold: %s G)' % (
      free_space, min_space))

  def test(self):
    return self._test(result_count=1, failure_amount=1)

  def anomaly(self):
    return self._test(result_count=3, failure_amount=3)



import requests
import re
import signal
import os
import psutil
import json
import time
import argparse

search_pid_regex = r"</td><td.*?>(.+?)</td><td>yes \(old gen\)</td>"

def loadJSONFile(db_path):
  if os.path.exists(db_path):
    with open(db_path) as json_file:
       try:
         return json.load(json_file)
       except ValueError:
         return {}
  else:
    return {}

def writeJSONFile(pid_dict, db_path):
  if db_path is None:
    # No place to save
    return 
  for pid in pid_dict.copy():
    try:
      process = psutil.Process(int(pid))
    except psutil.NoSuchProcess:
      del pid_dict[pid]

  with open(db_path, "w") as f:
    f.write(json.dumps(pid_dict))

def getServerStatus(url, user, password):
  try: 
    if user is not None:
      r = requests.get(url, auth=(user, password))
    else:
      r = requests.get(url)

    if r.status_code == 200:
      return r.text
  except requests.exceptions.ConnectionError:
    return 

def watchServerStatus(pid_dict, server_status, timeout):
  _pid_dict = pid_dict.copy()
  for i in re.findall(search_pid_regex, server_status):
    try:
      process = psutil.Process(int(i))
    except psutil.NoSuchProcess:
      continue

    # Ensure the process is actually an apache
    if process.cmdline()[0].endswith("/httpd"):
      _pid_dict.setdefault(i, time.time() + timeout)
      if _pid_dict[i] < time.time():
        print "Sending signal -%s to %s" % (signal.SIGKILL, i)
        try:
          process.kill()
        except psutil.NoSuchProcess:
          print "Process is not there anymore"
          continue

  return _pid_dict

def main():
  parser = argparse.ArgumentParser()
  # Address to ping to
  parser.add_argument("--url", required=True)
  # Force use ipv4 protocol
  parser.add_argument("-u", "--user")
  parser.add_argument("-p", "--password")
  parser.add_argument("-d", "--db")
  parser.add_argument("-t", "--timeout", default=600)
  args = parser.parse_args()

  pid_dict = loadJSONFile(args.db)

  server_status = getServerStatus(
    args.url, args.user, args.password)

  if server_status is None:
    raise ValueError("Couldn't connect to server status page")

  pid_dict = watchServerStatus(pid_dict, server_status, args.timeout)

  writeJSONFile(pid_dict, args.db)

